package com.ibm.iru.db2express9_1win;

/**
 * @(#)DB2Express9_1WinPDC.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */
import java.io.File;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportWindowsBase;
import com.ibm.jsdt.support.SupportWindowsHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * This class checks if DB2 Express 9.1 is installed on the system, as well
 * as other conditional variables.
 */
public class DB2Express9_1WinPDC extends SupportWindowsBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private String ivVersion = null;
  private Boolean migrate = false;
  private SupportWindowsHelper ivHelper;
  private DB2InstalledProduct db2Product;

  
  /**
   * Initializes the framework and validates arguments
   * Expects 1 arguments: 
   * @param arg[0] = response file name
   * @return SUCCESS = Initialization successful; 
   *         FAILURE = Arguments incorrect or other initialization failed 
   */
  private int init(String[] args)
  {
    int rc = SUCCESS;
    int no_Of_Expected_args = 1;
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getWindowsHelper();

    // Ensure the right number of parameters was passed
    if (no_Of_Expected_args == args.length)
    {
      setResponseFileName(args[0].trim());
      ivVersion = ivHelper.getProductInstallingVersion(this);
    }
    else
    {
      String[] replacementText = { "DB2Express9_1WinPDC", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      ivHelper.log(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      rc = FAILURE;
    }

    return rc;
  }
  
  /**
   * Perform required checks before installing DB2.
   * Need to check if installation directory specified exists and validate variables.
   * @return PDC_EXISTS, PDC_DOES_NOT_EXIST or FAILURE
   */
  private int performPredeploymentChecks()
  {
    int rc = determineInstallAction();
    
    // Check to see that the install will take place before validating deployment parameters. 
    if (rc != PDC_EXISTS && rc != FAILURE)
    {
      if(!migrate)
      {
        if (!validDeploymentParameters())
        {
          // override installAction
          rc = FAILURE;
        }
      }
      else
      {
        //if we are migrating, the DB2 copy name is the only parameter needed. The DB2 instance user and password
        //will be checked only if supplied, but are not required. 
        if(!validMigrationParameters())
        {
          rc = FAILURE;
        }
      }
    }
    return rc;
      
  }

  /**
   * determine install action:
   * 
   * @return PDC_DOES_NOT_EXIST - if specified install directory is non-existing or empty if existing
   *                            - if migrating from version 8
   *         PDC_EXIST -          if same version already installed
   *         FAILURE -            Another version of DB2 is already installed in the 
   *                              install directory, specified directory is not empty, or install location is not specified  
   */
  private int determineInstallAction()
  {
    int returnCode = PDC_DOES_NOT_EXIST;
    String versionInstalled = null;
    setKey("FILE");
    String installDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    
    setKey("MIGRATE_PRIOR_VERSIONS");
    String migratePriorVersion = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
        
    //if no value then return FAILURE as installation location is a required parameter
    if ((installDir == null) || installDir.equals(""))
    {
      setMessage(getResourceString(NLSKeys.INSTALLATION_LOCATION_NOT_FOUND));
      ivHelper.postErrorMessageToDeployerUI(this);    	
      return FAILURE;
    }
    
    //check if specified installation directory exists
    setFileName(installDir);
    if (ivHelper.fileExists(this))
    {
      //specified installation directory exists, check if it is empty
      File file = new File(installDir);
      if ((file.list()).length == 0)
      {
        //location is emtpy
        returnCode = PDC_DOES_NOT_EXIST;
      }
      else
      {
        //check if DB2 is installed 
        db2Product = DB2InstalledProduct.getInstalledProduct(this, installDir);
        if (db2Product == null)
        {
          setMessage(getResourceString(NLSKeys.NONEMPTY_DIR));
          ivHelper.postErrorMessageToDeployerUI(this);
          returnCode = FAILURE;
        }
        else
        {
          //get version installed up to fixpack level. Ex:9.1.0
          versionInstalled = db2Product.trimVersionToNVersionPlaces(3, db2Product.getInstalledVersion().trim());
          //check same version of DB2 is already installed
          if (versionInstalled.equals(ivVersion))
          {
            //Current version of DB2 is already installed
            returnCode = PDC_EXISTS;
          }
          else
          {
            //check if user selected to migrate from version 8, if so, then continue with the installation
            //otherwise return failure
            String versionInstalledOneDigit = db2Product.trimVersionToNVersionPlaces(1, versionInstalled);
            
            if (versionInstalledOneDigit.equals("8") && migratePriorVersion != null && migratePriorVersion.equalsIgnoreCase("YES"))
            {
              migrate = true;
              returnCode = PDC_DOES_NOT_EXIST;
              return returnCode;
            }
            setMessage(getResourceString(NLSKeys.OTHER_DB2_COPY_EXIST, versionInstalled));
            ivHelper.postErrorMessageToDeployerUI(this);
            //inform the users about the migrate option in the "Advanced" tab in the Deployment Wizard,
            //if they have version 8 installed
            if(versionInstalledOneDigit.equals("8"))
            {
              setMessage(getResourceString(NLSKeys.DB2_MIGRATE_OPTION));
              ivHelper.postInformationMessageToDeployerUI(this);
            }
            returnCode = FAILURE;
          }
        }
      }
    }
    return returnCode;
  }
  
  /**
   * Conditional and environmental variables are verified.
   * <li>1.Check that DB2 copy name specified is unique.
   * <li>2.Check that DB2 instance name specified is unique.
   * <li>3.Check that DB2 instance user name and password match (in case they exist).
   * <li>4.Check that if provided DAS user name and password, they match if they exist.
   * 
   * Any problems found will be logged.
   * 
   * @return true = no problems found. false = problem found.
   */
  private boolean validDeploymentParameters()
  {
    boolean validity = true;

    setKey("DB2_COPY_NAME");
    String db2CopyName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    if (db2CopyName == null || db2CopyName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_COPY_NAME));
      ivHelper.postErrorMessageToDeployerUI(this);
      validity = false;      
    }    
    setKey("DB2.NAME");
    String db2InstanceName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    if (db2InstanceName == null || db2InstanceName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_NAME));
      validity = false;
      ivHelper.postErrorMessageToDeployerUI(this);      
    }
    //check instance user name and password match
    setKey("DB2.USERNAME");
    String instanceUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("DB2.PASSWORD");
    String instanceUserPassword = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    if (instanceUserName == null || instanceUserName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_USER));
      validity = false;
      ivHelper.postErrorMessageToDeployerUI(this);      
    }
    
    if (instanceUserPassword == null ||  instanceUserPassword.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_INSTANCE_PASSWORD));
      validity = false;
      ivHelper.postErrorMessageToDeployerUI(this);
    }
    if(validity == true)
      validity = checkUserNamePassword(instanceUserName, instanceUserPassword);
    // Although DAS user configuration properties are required and this code is not needed, 
    // in the future these properties may not be compulsory and this section of code could
    // utilized again. 
    
    //check that if provided DAS user name and password, they match if they exist
    setKey("DAS_USERNAME");
    String dasUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("DAS_PASSWORD");
    String dasUserPass = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    if(dasUserName != null && !dasUserName.equals("") && dasUserPass != null && !dasUserPass.equals(""))
    {
      if(!checkUserNamePassword(dasUserName, dasUserPass))
        validity = false;
    } 
    
    if(validity == true)
    {
      //need to check that both DB2_COPY_NAME and DB2.NAME are unique
      DB2InstalledProduct [] db2Products = DB2InstalledProduct.getInstalledProducts(this);
      //for each DB2 product get the DB2 Copy Name and compare with the one given by the user
      if( db2Products != null)
      {
        for (int i=0; i < db2Products.length; i++)
        {
          String existingCopyName = db2Products[i].getDB2CopyName();
          if (existingCopyName != null && existingCopyName.equalsIgnoreCase(db2CopyName))
          {
            setMessage(getResourceString(NLSKeys.DB2_COPY_NAME_EXISTS, db2CopyName));
            validity = false;
            ivHelper.postErrorMessageToDeployerUI(this);
          }
          String[] existingInstanceNames = db2Products[i].getDB2InstanceNames();
          //It is possible that each DB2 copy have multiple instance names.
          if( existingInstanceNames != null)
          {            
            for (int j=0; j < existingInstanceNames.length; j++)
            {
              if (existingInstanceNames[j] != null && existingInstanceNames[j].equalsIgnoreCase(db2InstanceName))
              {
                setMessage(getResourceString(NLSKeys.DB2_INSTANCE_NAME_EXISTS, db2InstanceName));
                validity = false;
                ivHelper.postErrorMessageToDeployerUI(this);             
              }
            }            
          }          
        }        
      }
    }
    return (validity);
  }
  
  /**
   * For migration, only the DB2 Copy Name is required, and should be unique. This method
   * will check that DB2 Copy Name is unique. Also, if user specified a db2 instance user,
   * it will check that if the user exists, that the password matches. The DB2 instance user
   * is not a required field for migration, and thus it will not be a failure to leave blank.
   * @return true if DB2 Copy Name is unique, and if specified a DB2 instance user that exists
   * in the system, that its user id and password match. False otherwise.
   */
  private boolean validMigrationParameters()
  {
    boolean validity = true;

    setKey("DB2_COPY_NAME");
    String db2CopyName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    if (db2CopyName == null || db2CopyName.equals(""))
    {
      setMessage(getResourceString(NLSKeys.MISSING_DB2_COPY_NAME));
      ivHelper.postErrorMessageToDeployerUI(this);
      return false;      
    }
    
    // need to check that DB2_COPY_NAME is unique
    DB2InstalledProduct [] db2Products = DB2InstalledProduct.getInstalledProducts(this);
    //for each DB2 product get the DB2 Copy Name and compare with the one given by the user
    if(db2Products != null)
    {
      for (int i=0; i < db2Products.length; i++)
      {
        String existingCopyName = db2Products[i].getDB2CopyName();
        if (existingCopyName != null && existingCopyName.equalsIgnoreCase(db2CopyName))
        {
          setMessage(getResourceString(NLSKeys.DB2_COPY_NAME_EXISTS, db2CopyName));
          validity = false;
          ivHelper.postErrorMessageToDeployerUI(this);
          break;
        }
      }
    }
    
    // check if given instance user name and password, that they match if they exist
    setKey("DB2.USERNAME");
    String instanceUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("DB2.PASSWORD");
    String instanceUserPassword = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    if (instanceUserName != null && !instanceUserName.equals("") && instanceUserPassword != null &&  !instanceUserPassword.equals(""))
    {
      if (!checkUserNamePassword(instanceUserName, instanceUserPassword))
        validity = false;      
    }    
    
    return validity;
  }

  /**
   * Ensures the user ID and password match if they exist.
   *
   * @param String username, String password to check
   * @return true    - no user is found or the password for the user is correct
   *         false - the password is invalid for the given user.
   */
  private boolean checkUserNamePassword(String userName, String password)
  {
    Boolean validUserPassword = true;
    //Check the user
    setUserName(userName);
    if (getWindowsHelper().doesUserIdExist(this))
    {
      setPassword(password);
      if (!getWindowsHelper().isUserAndPasswordValid(this))
      {
        setMessage(getResourceString(NLSKeys.USERID_PASSWORD_MISMATCH, userName));
        getHelper().postErrorMessageToDeployerUI(this);
        validUserPassword = false;
      }      
    }   

    return validUserPassword;
  }

  /**
   * Expects the following parameter to be passed in:
   * parm#0 = response file name
   */
  public static void main(String args[])
  {
    // Create and initialize the object
    DB2Express9_1WinPDC pdc = new DB2Express9_1WinPDC();
    int retVal = pdc.init(args);

    // Check if DB2 is already installed and validate variables
    if (retVal == SUCCESS)
      retVal = pdc.performPredeploymentChecks();

    System.exit(retVal);
  }
}
